-------------------------------------------------------------------------
-- File Loader
--
-- $Id: FileLoader.lua,v 1.11 2005/08/17 14:26:15 okayu Exp $
-------------------------------------------------------------------------
LUA_PATH = "./?.lua;./LuaClass/?.lua"
require "LuaClass"

-- Class name declarations.
begin_declare()
declare("FileLoader")
declare("ImageLoader")
declare("ArchiveLoader")
declare("BuiltinImageLoader")
declare("BuiltinArchiveLoader")
declare("BuiltinBmpLoader")
declare("BuiltinJpegLoader")
declare("SPILoader")
declare("SPIArchiveLoader")
declare("CALArchiveLoader")


-------------------------------------------------------------------------
-- t@C[_[
-------------------------------------------------------------------------
FileLoader = { "FileLoader"; class = {}; }

	function FileLoader:init()
	end

	function FileLoader:isAvailable()
		return false
	end
	
	function FileLoader:fileTypeName()
		return ""
	end

	function FileLoader:loaderName()
		return ""
	end

	-- ̃t@CT|[gĂ邩H
	-- file : t@Cc[̃IuWFNg
	function FileLoader:isSupported( file )
		return false
	end

	function FileLoader:isImageLoader()
		return false
	end

	function FileLoader:isArchiveLoader()
		return false
	end

Class:new( FileLoader )


-------------------------------------------------------------------------
-- C[W[_[
-------------------------------------------------------------------------
ImageLoader = { "ImageLoader", "FileLoader"; class = {}; }

	function ImageLoader:init()
		self:super()
	end
	
	-- 摜obt@擾
	-- Ffile
	-- ߂lFinfoBuffer, bitBuffer
	ImageLoader.getPicture = Class.subclassResponsibility

	function ImageLoader:isImageLoader()
		return true
	end

Class:new( ImageLoader )


-------------------------------------------------------------------------
-- A[JCu[_[
-------------------------------------------------------------------------
ArchiveLoader = { "ArchiveLoader", "FileLoader"; class = {}; }

	function ArchiveLoader:init()
		self:super()
	end

	-- t@Cobt@擾
	-- Ffile
	-- ߂lFinfoBuffer, bitBuffer
	ArchiveLoader.getFile = Class.subclassResponsibility;

	function ArchiveLoader:isArchiveLoader()
		return true
	end

Class:new( ArchiveLoader )


-------------------------------------------------------------------------
-- {̓[_[
-------------------------------------------------------------------------
BuiltinImageLoader = { "BuiltinImageLoader", "ImageLoader"; class = {}; }

	function BuiltinImageLoader:init()
		self:super()
	end

	function BuiltinImageLoader:loaderName()
		return "{"
	end

Class:new( BuiltinImageLoader )


BuiltinArchiveLoader = { "BuiltinArchiveLoader", "ArchiveLoader"; class = {}; }

	function BuiltinArchiveLoader:init()
		self:super()
	end

	function BuiltinArchiveLoader:loaderName()
		return "{"
	end

Class:new( BuiltinArchiveLoader )


-- rgCBMP[_[
BuiltinBmpLoader = { "BuiltinBmpLoader", "BuiltinImageLoader"; class = {}; }

	function BuiltinBmpLoader:init()
 		self:super()
 		FileType:registerLoader( self, "*.bmp;*.dib" )
	end

	function BuiltinBmpLoader:isAvailable()
		return true
	end
	
	function BuiltinBmpLoader:fileTypeName()
		return "rbg}bv"
	end

	function BuiltinBmpLoader:isSupported( file )
		if file:fsFilePath() then
			return app.isSupportedBmp( file:fsFilePath() )
		end
		local buffer = file:cachedBuffer()
		if buffer == nil then return false end
		local ret = app.isSupportedBmp( file:path(), buffer )
		return ret
	end

	-- [h
	-- Fbuffer  t@Cǂݍ񂾃obt@
	-- ߂lFinfoBuffer, bitBuffer
	function BuiltinBmpLoader:getPicture( buffer )
		return app.loadBmp( buffer )
	end

Class:new( BuiltinBmpLoader )


-- rgCJPEG[_[
BuiltinJpegLoader = { "BuiltinJpegLoader", "BuiltinImageLoader"; class = {}; }

	function BuiltinJpegLoader:init()
 		self:super()
 		FileType:registerLoader( self, "*.jpg;*.jpeg" )
	end

	function BuiltinJpegLoader:isAvailable()
		return true
	end
	
	function BuiltinJpegLoader:fileTypeName()
		return "JPEG"
	end

	function BuiltinJpegLoader:isSupported( file )
		if file:fsFilePath() then
			return app.isSupportedJpeg( file:fsFilePath() )
		end
		local buffer = file:cachedBuffer()
		if buffer == nil then return false end
		local ret = app.isSupportedJpeg( file:path(), buffer )
		return ret
	end

	-- [h
	-- Fbuffer  t@Cǂݍ񂾃obt@
	-- ߂lFinfoBuffer, bitBuffer
	function BuiltinJpegLoader:getPicture( buffer )
		return app.loadJpeg( buffer )
	end

Class:new( BuiltinJpegLoader )



-------------------------------------------------------------------------
-- roh[_[
-------------------------------------------------------------------------
SPILoader = { "SPILoader", "ImageLoader"; class = {}; }

	function SPILoader:init( pluginPath, plugin )
		self:super()
		self.mIsAvailable = true
		self.mCopyright = plugin:getPluginInfo(2)
		self.mExtensions = plugin:getPluginInfo(3)
		self.mFileTypeName = plugin:getPluginInfo(4)
		self.mPluginPath = pluginPath
		self.mLoadedPlugin = plugin
		
		-- vOCʑΉ
		if self.mCopyright == "ICO/CUR to DIB filter ver 0.10 (c)ExpF" then
			-- ̃vOC "*.cur" Ήgq甲Ă̂Œǉ
			self.mExtensions = "*.ico;*.cur"
		end

		print( "Check susie image plugin :" ) 
		print( "   "..pathToFilename(pluginPath).." ( "..self.mExtensions.." )" )
		print( "   "..self.mCopyright )

		SPILoader:addCachedLoader( self )
		FileType:registerLoader( self, self.mExtensions )
	end

	function SPILoader:isAvailable()
		return self.mIsAvailable
	end
	
	function SPILoader:fileTypeName()
		return self.mFileTypeName
	end

	function SPILoader:loaderName()
		return self.mCopyright
	end

	-- ̃tH[}bgT|[gĂ邩H
	-- Ffile  t@C
	function SPILoader:isSupported( file )
		local spiPlugin = self:plugin()
		local buffer = file:cachedBuffer()

		if spiPlugin == nil then return end
		if buffer == nil then return end

		local ret = spiPlugin:isSupported( file:path(), buffer )
		return ret
	end

	-- 摜擾
	-- Fbuffer  t@Cǂݍ񂾃obt@
	-- ߂lFe[u
	function SPILoader:getPictureInfo( buffer )
		if self:plugin() == nil then return end
		return self.mLoadedPlugin:getPictureInfo( buffer )
	end

	-- [h
	-- Fbuffer  t@Cǂݍ񂾃obt@
	-- ߂lFinfoBuffer, bitBuffer
	function SPILoader:getPicture( buffer )
		if self:plugin() == nil then return end
		return self.mLoadedPlugin:getPicture( buffer )
	end

	-- pBSusievOC[hĕԂ
	function SPILoader:plugin()
		if self.mLoadedPlugin == nil then
			self.mLoadedPlugin = app.loadSusiePlugin( self.mPluginPath )
			SPILoader:addCachedLoader( self )
		end
		return self.mLoadedPlugin
	end

	------------- NX\bh -------------

	function SPILoader.class:init()
		-- vOCLbVĂ郍[_[̃Xg
		self.mCachedLoaders = Array()
		-- LbVĂSusievOC̐
		self.mMaxCachedPlugin = 5
	end
	
	function SPILoader.class:addCachedLoader( loader )
		if loader.mLoadedPlugin == nil then return end
		
		-- ̃[_[Xg̍Ōɕt
		local i = self.mCachedLoaders:find( loader )
		if i then self.mCachedLoaders:remove( i ) end
		self.mCachedLoaders:push( loader )
		
		-- 萔ȏɂȂ烊Xg̐擪̃vOCA[hĂ
		while self.mCachedLoaders:size() > self.mMaxCachedPlugin do
			local oldLoader = self.mCachedLoaders:popFront()
			if oldLoader.mLoadedPlugin then
				oldLoader.mLoadedPlugin:unload()
				oldLoader.mLoadedPlugin = nil
			end
		end
	end
	
Class:new( SPILoader )



-------------------------------------------------------------------------
-- rohA[JCu[_[
-------------------------------------------------------------------------
SPIArchiveLoader = { "SPIArchiveLoader", "ArchiveLoader"; class = {}; }

	function SPIArchiveLoader:init( pluginPath, plugin )
		self:super()
		self.mIsAvailable = true
		self.mCopyright = plugin:getPluginInfo(2)
		self.mExtensions = plugin:getPluginInfo(3)
		self.mFileTypeName = plugin:getPluginInfo(4)
		self.mPluginPath = pluginPath
		self.mLoadedPlugin = plugin
		
		print( "Check susie archive plugin :" ) 
		print( "   "..pathToFilename(pluginPath).." ( "..self.mExtensions.." )" )
		print( "   "..self.mCopyright )

		SPIArchiveLoader:addCachedLoader( self )
		FileType:registerLoader( self, self.mExtensions )
	end

	function SPIArchiveLoader:isAvailable()
		return self.mIsAvailable
	end
	
	function SPIArchiveLoader:fileTypeName()
		return self.mFileTypeName
	end

	function SPIArchiveLoader:loaderName()
		return self.mCopyright
	end

	-- ̃tH[}bgT|[gĂ邩H
	-- Ffile  A[JCut@C
	function SPIArchiveLoader:isSupported( file )
		local spiPlugin = self:plugin()
		local buffer = file:cachedBuffer()

		if spiPlugin == nil then return end
		if buffer == nil then return end

		local ret = spiPlugin:isSupported( file:path(), buffer )
		return ret
	end

	-- A[JCut@C̃t@C̃Xg擾
	-- FarchiveFile  A[JCut@C
	function SPIArchiveLoader:getFileList( archiveFile )
		print( "SPIArchiveLoader:getFileList()" )
		local spiPlugin = self:plugin()
		if spiPlugin == nil then
			return
		end
		local infoList, fileCount, err
		err = ""
		local startTime = app.getTicks()
		
		-- obt@擾
		local archiveBuffer = archiveFile:cachedBuffer()
		if archiveBuffer then
			infoList, fileCount, err = spiPlugin:getArchiveInfo( archiveBuffer )
		end
		
		if err ~= nil then
			if archiveFile:fsFilePath() then
				-- t@CVXeɂꍇ̓t@C擾
				print( "SPIArchiveLoader:getFileList() "..archiveFile:fsFilePath() )
				infoList, fileCount, err = spiPlugin:getArchiveInfo( archiveFile:fsFilePath() )
			end
		end

		print( "Xg쐬ԁF"..( (app.getTicks() - startTime) / 1000 ).."b" )
		return infoList or {}, fileCount or 0, err
	end

	-- A[JCũt@Co
	--     file  A[JCut@C
	-- ߂l  ot@Cobt@
	function SPIArchiveLoader:getFile( file )
		local spiPlugin = self:plugin()
		if spiPlugin == nil then return end
		
		-- vOXEChȄ
--~ 		local progressWindow = app.createProgressWindow( cancellCallback )
--~ 		local doesUserCancel = false
--~ 		local cancellCallback = function ()
--~ 			print( "cancellCallback" )
--~ 			doesUserCancel = true
--~ 		end
--~ 		local callbackFunc = function( rate )
--~ 			print( "callbackFunc : "..rate )
--~ 			progressWindow:setProgressPos( rate )
--~ 			progressWindow:pumpMessages()
--~ 			if doesUserCancel then
--~ 				progressWindow:close()
--~ 			end
--~ 		end

--~ 		progressWindow:setTitle( "A[JCu̓WJ" )
--~ 		progressWindow:setMessage( file:name().." ..." )
--~ 		progressWindow:show()
		
		-- obt@ǂݍ
		local ret
		local buffer = file:containerFile():cachedBuffer()
		if buffer then
			ret = spiPlugin:getFile(
				buffer,
				file.mExpandInfo.position,
				file.mExpandInfo.filesize,
				callbackFunc )
		end

		if ret == nil then
			-- obt@̓ǂݍ݂ɑΉĂȂvOC
			if file:containerFile():fsFilePath() then
				-- Reit@Ct@CVXeɂꍇ͒ړǂݍ
				ret = spiPlugin:getFile(
					file:containerFile():fsFilePath(),
					file.mExpandInfo.position,
					file.mExpandInfo.filesize,
					callbackFunc )
			end
		end

		-- vOXEChE
--~ 		progressWindow:close()
		
		return ret
	end


	-- pBSusievOC[hĕԂ
	function SPIArchiveLoader:plugin()
		if self.mLoadedPlugin == nil then
			self.mLoadedPlugin = app.loadSusiePlugin( self.mPluginPath )
			SPIArchiveLoader:addCachedLoader( self )
		end
		self.mIsAvailable = (self.mLoadedPlugin ~= nil)
		return self.mLoadedPlugin
	end

	
	------------- NX\bh -------------

	function SPIArchiveLoader.class:init()
		-- vOCLbVĂ郍[_[̃Xg
		self.mCachedLoaders = Array()
		-- LbVĂSPI摜vOC̐
		self.mMaxCachedPlugins = 5
	end
	
	function SPIArchiveLoader.class:addCachedLoader( loader )
		if loader.mLoadedPlugin == nil then return end
		
		-- ̃[_[Xg̍Ōɕt
		local i = self.mCachedLoaders:find( loader )
		if i then self.mCachedLoaders:remove( i ) end
		self.mCachedLoaders:push( loader )
		
		-- 萔ȏɂȂ烊Xg̍ŏ̃vOC̓A[h
		if self.mCachedLoaders:size() > self.mMaxCachedPlugins then
			local oldLoader = self.mCachedLoaders:popFront()
			if oldLoader.mLoadedPlugin then
				oldLoader.mLoadedPlugin:unload()
				oldLoader.mLoadedPlugin = nil
			end
		end
	end
	
Class:new( SPIArchiveLoader )


-------------------------------------------------------------------------
-- A[JCoCu[_[
-------------------------------------------------------------------------
CALArchiveLoader = { "CALArchiveLoader", "ArchiveLoader"; class = {}; }

	function CALArchiveLoader:init()
		self:super()
		self.mPluginPath = nil		-- vOCpX
		self.mPluginInfo = nil		-- 
	end

Class:new( CALArchiveLoader )


-------------------------------------------------------------------------
-- end of file
-------------------------------------------------------------------------
end_declare()