-------------------------------------------------------------------------
-- Tree collection
--
-- $Id: Tree.lua,v 1.11 2005/07/12 00:55:24 okayu Exp $
-------------------------------------------------------------------------
LUA_PATH = "./?.lua;./LuaClass/?.lua"
require "LuaClass"

-- Class name declarations.
begin_declare()
declare("TreeNode")
declare("TreeLeaf")
declare("TreeBranch")
declare("Tree")



-------------------------------------------------------------------------
-- Tree node class (abstract)
-------------------------------------------------------------------------
TreeNode = { "TreeNode"; class = {}; }

	-- instance variables : mParent


	-- instance methods.

	function TreeNode:init()
--~ 		self.mParent = nil
--~ 		self.mIsMarked = false
	end
	
	
	function TreeNode:isRoot()
		return self.mParent == nil
	end

	TreeNode.isBranch = Object.subclassResponsibility
	
	TreeNode.isLeaf = Object.subclassResponsibility

	function TreeNode:isParent( node )
		return self.mParent == node
	end

	function TreeNode:isChild( node )
		return false
	end	

	function TreeNode:isSibling( node )
		return self.mParent == node:getParent()
	end

	function TreeNode:isAncestor( branch )
		local parent = self:getParent()
		while parent do
			if parent == branch then return true end
			parent = parent:getParent()
		end
		return false
	end

	function TreeNode:isDescendant( node )
		return false
	end
	
	function TreeNode:getParent()
		return self.mParent
	end
	
	function TreeNode:getRoot()
		local node = self
		while not node:isRoot() do
			node = node:getParent()
		end
		return node
	end
	
	function TreeNode:isMarked()
		return self.mIsMarked
	end
	
	function TreeNode:setMark( bool )
		if bool == nil then bool = true end
		self.mIsMarked = bool
	end
	
	
	function TreeNode:getNodeCount()
		return 0
	end

	function TreeNode:getTotalNodeCount()
		return 0
	end

	function TreeNode:getMarkCount()
		return 0
	end

	function TreeNode:getTotalMarkCount()
		return 0
	end

	
	function TreeNode:getNodeIndex()
		if self.mParent == nil then return 1 end
		return self.mParent:getChildNodeIndex( self )
	end

	function TreeNode:getNodeDepth()
		local count = 0
		local parent = self.mParent
		while parent do
			count = count + 1
			parent = parent:getParent()
		end
		return count
	end
	
	
	function TreeNode:foreachAncestor( fn )
		local parent = self:getParent()
		local tmp
		while parent do
			tmp = fn(nil, parent)
			if tmp ~= nil then return tmp end
			parent = parent:getParent()
		end
		return nil
	end

	TreeNode.foreachChild = Object.subclassResponsibility

	TreeNode.foreachDescendant = Object.subclassResponsibility
	
	TreeNode.foreachReverseDescendant = Object.subclassResponsibility

	
	
	-- return a next node in the root tree.
	function TreeNode:nextNode()

		local findNode = function( i, v ) return v end
		
		-- search descendants
		if self:isBranch() then
			local node = self:foreachDescendant( findNode )
			if node then return node end
		end
		
		local parent = self:getParent()
		if parent == nil then
			return nil
		end
	
		-- search siblings
		local i
		local countMax = parent:getNodeCount()
		for i = self:getNodeIndex() + 1, countMax do
			local node = parent:getChildAt( i )
			if node then
				return node
			end
		end
	
		-- search parent's siblings
		local node
		repeat
			local target = parent:getParent()
			if target == nil then
				return nil
			end
			node = target:getChildAt( parent:getNodeIndex() + 1 )
			parent = target
		until node ~= nil

		return node
	end
	
	
	-- return a prev node in the root tree.
	function TreeNode:prevNode()
		
		local findNode = function( i, v ) return v end

		local parent = self:getParent()
		if parent == nil then
			return nil
		end
	
		-- search descendants
		if self:isBranch() then
			local node = self:foreachReverseDescendant( findNode )
			if node then return node end
		end
		
		-- search siblings
		local i
		for i = self:getNodeIndex() - 1, 1, -1 do
			local node = parent:getChildAt( i )
			if node then
				return node
			end
		end
	
		-- search ancestor's siblings
		local node
		repeat
			local target = parent:getParent()
			if target == nil then
				return nil
			end
			node = target:getChildAt( parent:getNodeIndex() - 1 )
			parent = target
		until node ~= nil
		
		return node
	end


	-- class method.
	TreeNode.class.new = Object.notSupported;

Class:new( TreeNode )



-------------------------------------------------------------------------
-- TreeLeaf class
-------------------------------------------------------------------------
TreeLeaf = { "TreeLeaf", "TreeNode"; }

	-- instance variables :

	function TreeLeaf:init()
		self:super()
	end


	-- instance methods.

	function TreeLeaf:isBranch()
		return false
	end

	function TreeLeaf:isLeaf()
		return true
	end

Class:new( TreeLeaf )



-------------------------------------------------------------------------
-- TreeBranch class
-------------------------------------------------------------------------
TreeBranch = { "TreeBranch", "TreeNode"; }

	-- instance variables :
	--    mChildren, mTotalNodeCount, mMarkCount, mTotalMarkCount
	
	function TreeBranch:init()
		self:super()
		self.mChildren = Array()
		self.mTotalNodeCount = 0
		self.mMarkCount = 0
		self.mTotalMarkCount = 0
	end

	function TreeBranch:isBranch()
		return true
	end

	function TreeBranch:isLeaf()
		return false
	end

	function TreeBranch:isChild( node )
		return self.mChildren:hasValue( ndoe )
	end
	
	function TreeBranch:isDescendant( node )
		return node:isAncestor(self)
	end
	
	function TreeBranch:getChildAt( index )
		return self.mChildren:at( index )
	end


	function TreeBranch:getNodeCount()
		return self.mChildren:size()
	end

	function TreeBranch:getTotalNodeCount()
		return self.mTotalNodeCount
	end

	function TreeBranch:getMarkCount()
		return self.mMarkCount
	end

	function TreeBranch:getTotalMarkCount()
		return self.mTotalMarkCount
	end

	
	function TreeBranch:foreachChild( fn )
		return self.mChildren:foreach( fn )
	end

	function TreeBranch:foreachDescendant( fn )
		return self.mChildren:foreach(
			function(i, v)
				local tmp = fn( nil, v )
				if tmp then return tmp end
				if v:isBranch() then
					tmp = v:foreachDescendant( fn )
					if tmp then return tmp end
				end
			end
		)
	end

	function TreeBranch:foreachReverseDescendant( fn )
		return self.mChildren:reverse():foreach(
			function(i, v)
				local tmp
				if v:isBranch() then
					tmp = v:foreachReverseDescendant( fn )
					if tmp then return tmp end
				end
				tmp = fn( nil, v )
				if tmp then return tmp end
			end
		)
	end

	
	function TreeBranch:addNode( node )
		if node.mParent == self then return end
		self:increaseNodeCount( node )
		self.mChildren:add( node )
		if node.mParent then
			node.mParent:removeNode( node )
		end
		node.mParent = self
	end

	function TreeBranch:insertNode( pos, node )
		if node.mParent == self then return end
		pos = math.max( 1, math.min(pos, self.mChildren:size() + 1) )
		self:increaseNodeCount( node )
		self.mChildren:insert( pos, node )
		if node.mParent then
			node.mParent:removeNode( node )
		end
		node.mParent = self
	end

	function TreeBranch:removeNode( node )
		if node.mParent ~= self then return end
		local index = self.mChildren:find( node )
		if index then
			self.mChildren:remove( index )
		end
		self:decreaseNodeCount( node )
		node.mParent = nil
	end

	
	-- private methods.

	function TreeBranch:getChildNodeIndex( node )
		return self.mChildren:find( node )
	end

	
	function TreeBranch:updateMarkCountBy( num )
		self.mMarkCount = self.mMarkCount + num
	end

	function TreeBranch:updateTotalMarkCountBy( num )
		self.mTotalMarkCount = self.mTotalMarkCount + num
		if self.mParent then
			self.mParent:updateTotalMarkCountBy( num )
		end
	end

	function TreeBranch:updateTotalNodeCountBy( num )
		self.mTotalNodeCount = self.mTotalNodeCount + num
		if self.mParent then
			self.mParent:updateTotalNodeCountBy( num )
		end
	end

	function TreeBranch:increaseNodeCount( node )
		local marked = node:getTotalMarkCount()
		if node:isMarked() then
			marked = marked + 1
			self:updateMarkCountBy( 1 )
		end
		self:updateTotalMarkCountBy( marked )
		self:updateTotalNodeCountBy( node:getTotalNodeCount() + 1 )
	end

	function TreeBranch:decreaseNodeCount( node )
		local marked = node:getTotalMarkCount()
		if node:isMarked() then
			marked = marked + 1
			self:updateTotalMarkCountBy( -1 )
		end
		self:updateTotalMarkCountBy( - marked )
		self:updateTotalNodeCountBy( - (node:getTotalNodeCount() + 1) )
	end

Class:new( TreeBranch )



-------------------------------------------------------------------------
-- Tree class
--
-- tree = Tree { 1, 2, 3, 
-------------------------------------------------------------------------
Tree = { "Tree", "Collection"; class = {} }

	-- instance variables : mRoot


	-- instance methods.

	function Tree:init()
		self.mRoot = TreeBranch()
		
	end
	


	-- return the number of items.
	Tree.size = Object.subclassResponsibility;

	-- do for all items.
	Tree.foreach = Object.subclassResponsibility;

	--  add
	Tree.add	 = Object.subclassResponsibility;


	function Tree:isEmpty()
		return self:size() == 0
	end

	-- create string to represent no key collection info.
	function Tree:tostring()
		local str = "{ "
		local firstTime = true
		-- enumurate all items.
		self:foreach(
			function(i, v)
				if firstTime then firstTime = false
				else str = str..", "
				end
				str = str..LuaClass.objectDigest(v)
			end
		)
		return str.." }"
	end

	-- has some value?
	function Tree:hasValue( value )
		return self:find(value) ~= nil
	end

	-- find value.
	function Tree:find( value )
		return self:foreach(
			function(i, v)
				if v == value then return value end
			end
		)
	end

	-- class method.
	Tree.class.new = Object.notSupported;

Class:new( Tree )



-------------------------------------------------------------------------
-- end of file
-------------------------------------------------------------------------
end_declare()