-------------------------------------------------------------------------
-- bind
--
-- make closure which binds arguments to some function.
--
-- $Id: bind.lua,v 1.4 2005/05/30 00:46:39 okayu Exp $
-------------------------------------------------------------------------
require "declare"


-- Function name declarations.
begin_declare()
declare("bind")
declare("bindLeft")
declare("bindRight")
declare("bindself")
declare("bindselfLeft")
declare("bindselfRight")

local combine_array


-------------------------------------------------------------------------
-- bindLeft
--
-- bind arguments to left.
-- like bind1st in STL.
-------------------------------------------------------------------------
function bindLeft(func, ...)
	assert( type(func) == "function", "function expected" )

	local bound_arg = arg
	local ret = function (...)
		return func( unpack( combine_array(bound_arg, arg) ) )
	end
	return ret
end


-------------------------------------------------------------------------
-- bindRight
--
-- bind arguments to right.
-- like bind2nd in STL.
-------------------------------------------------------------------------
function bindRight(func, ...)
	assert( type(func) == "function", "function expected" )

	local bound_arg = arg
	local ret = function (...)
		return func( unpack( combine_array(arg, bound_arg) ) )
	end
	return ret
end


-------------------------------------------------------------------------
-- bind
--
-- same as bindLeft.
-------------------------------------------------------------------------
bind = bindLeft


-------------------------------------------------------------------------
-- bindselfLeft
--
-- bind also self object to the left-binding closure.
-------------------------------------------------------------------------
function bindselfLeft(self, func, ...)
	assert( type(self) == "table" or type(self) == "userdata",
		"table or userdata expected" )
	return bind( bind(func, self), unpack(arg) )
end


-------------------------------------------------------------------------
-- bindselfRight
--
-- bind also self object to the right-binding closure.
-------------------------------------------------------------------------
function bindselfRight(self, func, ...)
	assert( type(self) == "table" or type(self) == "userdata",
		"table or userdata expected" )
	return bindRight( bind(func, self), unpack(arg) )
end


-------------------------------------------------------------------------
-- bindself
--
-- same as bindselfLeft.
-------------------------------------------------------------------------
bindself = bindselfLeft


-------------------------------------------------------------------------
-- local function
-------------------------------------------------------------------------
-- combine_array()
-- 
-- Combine tables as array.
-- This function will do re-number index,
-- and ignore items which indexed by not number.
-------------------------------------------------------------------------
function combine_array(t1, t2)
	assert( type(t1) == "table" and type(t2) == "table" )

	local t = { unpack(t1) }
	local size1 = table.getn( t1 )
	local size2 = table.getn( t2 )
	local i
	for i = 1, size2 do
		t[size1 + i] = t2[i]
	end
	return t
end



-------------------------------------------------------------------------
-- end of file
-------------------------------------------------------------------------
end_declare()