-------------------------------------------------------------------------
-- declare.lua
--
-- enforce declarations of global variables.
--
-- You don't have to use this hack.
-- It makes safety from miss-typing, but the error message is esoteric.
-- This is used principally for debugging internal luaClass.
--
-- It affects only between begin_declare() to end_declare().
-- If you call begin_declare(), declaration is enforced.
-- Without declaration, you are denied to access global variables.
-- Call end_declare(), and you can access.
--
-- note:
--   You cannot use `require' after `begin_declare()' in your main script.
--   `require' internaly reads a global variable `_REQUIREDNAME'.
--   But it's not defined in main script, `require' raise an error.
--
-- $Id: declare.lua,v 1.5 2005/06/13 12:38:11 okayu Exp $
-------------------------------------------------------------------------
local isDeclareForced = false

-- declare( var_name )
function declare( var_name )
	assert( type(var_name) == "string", "declare() : "..
		"string argument is expected." )
	
	if isDeclareForced then
		rawset(_G, var_name, false)	-- initialized by `false' instead of nil.
	end
end

-- begin force declaration.
function begin_declare()
	setmetatable( _G, {
		__index = 0;		-- reading global var raise this error:
		                    --   "attempt to index a number value"
		__newindex = "";    -- writting to global var raise this error:
							--   "attempt to index a string value"
	} )
	isDeclareForced = true
end

-- end force declaration.
function end_declare()
	setmetatable( _G, nil )
end


--------- for ignore declarations ---------
-- function declare() end
-- function begin_declare() end
-- function end_declare() end
-------------------------------------------

-------------------------------------------------------------------------
-- end of file
-------------------------------------------------------------------------