-------------------------------------------------------------------------
-- Resource
--
-- $Id: Resource.lua,v 1.12 2005/08/17 14:26:15 okayu Exp $
-------------------------------------------------------------------------
LUA_PATH = "./?.lua;./LuaClass/?.lua"
require "LuaClass"

-- Class name declarations.
begin_declare()
declare("Resource")

declare("AbstractFile")
declare("File")
declare("ArchivedFile")

declare("AbstractDirectory")
declare("Directory")
declare("ArchivedDirectory")
declare("Folder")


-------------------------------------------------------------------------
-- ۃ\[XNX
--
-- t@CfBNgȂǁAt@CXgɒǉłΏۂ\NX
-- ׂẴ\[X͎̉Ƀc[\邱Ƃł
-------------------------------------------------------------------------
Resource = { "Resource", "TreeBranch"; class = {} }

	function Resource:init( name, path )
		self:super()
		if name then self:setName( name ) end
		if path then self:setPath( path ) end
		self.mIsSubTreeOpened = false
	end

	-- \[Xݒ肷
	function Resource:setName( name )
		self.mName = name
	end

	-- \[XpXݒ肷
	function Resource:setPath( path )
		self.mPath = path
		-- OݒȂpXt@C؂oĐݒ
		if self.mName == nil then
			local dir, name = app.splitPath( path )
			self:setName( name )
		end
	end

	-- Xg擪̃CfbNXԍvZĕԂ
	function Resource:resourceIndex()
		return self:countPrevNodes()
	end


	-- LbVtobt@
	function Resource:cachedBuffer( callback )
		if self:hasArchiveLoader() then
			-- A[JCut@C̃obt@LbV
			local cache = gData.archiveBufferCache:at( self )
			if cache then
				return cache
			end
			local buffer = self:buffer( callback )
			gData.archiveBufferCache:add( self, buffer )
			return buffer
		end
		-- ʂ̃obt@LbV
		local cache = gData.bufferCache:at( self )
		if cache then return cache end
		local buffer = self:buffer( callback )
		gData.bufferCache:add( self, buffer )
		return buffer
	end
	
	-- ̃\[Xɓǂݍ񂾃obt@Ԃ
	function Resource:buffer()
		return nil
	end

	-- ̃t@C܂ރA[JCut@C
	function Resource:containerFile()
		return nil
	end


	-------------------------------------------------------------
	-- {IȃvpeB
	-------------------------------------------------------------
	-- \[X
	function Resource:name()
		return self.mName or ""
	end

	-- tpX
	function Resource:path()
		return self.mPath or ""
	end

	-- TCY
	function Resource:size()
		return 0
	end

	-- f
	-- cx, cy, bpp = resource:pixelInfo()
	function Resource:pixelInfo()
		return 0, 0, 0
	end

	-- t@C^Cv
	function Resource:fileType()
		return nil
	end

	-- t@CH
	function Resource:isFile()
		return false
	end

	-- fBNgH
	function Resource:isDirectory()
		return false
	end

	-- Kwꂽ\[XH
	function Resource:hasSubTree()
		return false
	end

	-- Tuc[͓ǂݍ܂Ă邩
	function Resource:isSubTreeLoaded()
		return false
	end

	-- Tuc[t@CVXeǂݍ
	function Resource:loadSubTree()
		return false
	end
	
	
	-------------------------------------------------------------
	-- r[\֘A
	-------------------------------------------------------------

	-- ̃\[X͕\Ă邩H
	function Resource:isShown()
		local foundClosed = self:foreachAncestor(
			function( i, v )
				if v:isSubTreeClosed() then return v end
			end
		)
		if foundClosed then return false end
		return true
	end

	function Resource:isHidden()
		return not self:isShown()
	end

	-- Tuc[͕\I[vĂ邩
	function Resource:isSubTreeOpened()
		return self.mIsSubTreeOpened
	end

	-- Tuc[͕\N[YĂ邩
	function Resource:isSubTreeClosed()
		return not self.mIsSubTreeOpened
	end
	
	-- Tuc[̊JԂݒ
	function Resource:setSubTreeOpenFlag( isOpened )
		self.mIsSubTreeOpened = isOpened
	end
	
	-- Tuc[ŊJĂ郊\[Xs̔zԂ
	-- row : ̃\[X̍sԍivZȂ߁Bȗj
	function Resource:getShownRows( row )
		row = row or self:resourceIndex()
		local tbl = Array()
		self:_getShownRowsSub( row, tbl )
		return tbl
	end
	
	function Resource:_getShownRowsSub( row, tbl )
		local row = row
		tbl:add( {row, 1} )

		-- Tuc[ȂꍇAĂꍇ͖߂
		if not self:hasSubTree() then return end
		if self:isSubTreeClosed() then return end
		
		-- JĂꍇ̓Tuc[̊em[h₢킹
		row = row + 1
		self.mChildren:foreach(
			function( i, v )
				local count = v:getTotalNodeCount() + 1
				v:_getShownRowsSub( row, tbl )
				row = row + count
			end
		)
	end
	
	-------------------------------------------------------------
	-- [_[֘A
	-------------------------------------------------------------
	-- ̃\[X̉摜[_[܂ nil Ԃ
	function Resource:imageLoader()
		return nil
	end

	-- ̃\[X̃A[JCu[_[܂ nil Ԃ
	function Resource:archiveLoader()
		return nil
	end

	-- ̃\[X̉摜[_[݂΁Atrue Ԃ
	function Resource:hasImageLoader()
		return false
	end

	-- ̃\[X̃A[JCu[_[݂΁Atrue Ԃ
	function Resource:hasArchiveLoader()
		return false
	end
	
	-------------------------------------------------------------
	-- VACY
	-------------------------------------------------------------
	-- fVACY
	function Resource:deserialize( data, basePath )
		-- e[u珉
	end

	-- VACY
	-- stream   : io.open() ŊJꂽt@Cnh
	-- basePath : ΃pXŏoꍇ̓x[XpXB΃pX̏ꍇ nil
	-- indent   : Cfg
	function Resource:serialize( stream, basePath, indent )
		local spc = string.rep("  ", indent)
		stream:write(spc..'{ class="Resource" };\n')
	end


	-------------------------------------------------------------
	-- \[X̒ǉ
	-------------------------------------------------------------
	-- addNode ̃I[o[Ch
	function Resource:addNode( node )
		self:super( node )
		self:getRoot():onAddDescendant( node )
	end
	
	-- \[Xǉ̏i[g̃m[ĥݎ󂯎j
	function Resource:onAddDescendant( node )
	end
	
	
	-------------------------------------------------------------
	-- ̑
	-------------------------------------------------------------
	-- \[Xc[ł̑Õ}[Nꂽm[hvZ
	function Resource:countPrevMarks()
		local count = 0
		local parent = self:getParent()
		if parent == nil then return 0 end

		local i
		for i = self:getNodeIndex() - 1, 1, -1 do
			local sibling = parent:getChildAt(i)
			if sibling then
				if sibling:isMarked() then count = count + 1 end
				count = count + sibling:getTotalMarkCount()
			end
		end
		return count + parent:countPrevMarks()
	end

	-- \[Xc[ł̑ÔׂẴm[hvZ
	function Resource:countPrevNodes()
		local count = 0
		local parent = self:getParent()
		if parent == nil then return 0 end

		local i
		for i = self:getNodeIndex() - 1, 1, -1 do
			local sibling = parent:getChildAt(i)
			if sibling then
				count = count + 1
				count = count + sibling:getTotalNodeCount()
			end
		end
		return count + parent:countPrevNodes() + 1
	end

	-- c[ł̎wCfbNX̃\[XԂ
	function Resource:getResourceAt( index, count )
		local count = count or 0

		-- t@C̏ꍇ
		if index == count then return self end

		-- ̃c[̉wm[hT
		local i
		for i=1, self:getNodeCount() do
			local childs = self:getChildAt(i):getTotalNodeCount()
			count = count + 1
			if index <= count + childs then
				-- ړĨm[h͂̎qm[hɂ
				return self:getChildAt(i):getResourceAt( index, count )
			end
			count = count + childs
		end

		-- Ȃ
		return nil
	end
	
	-- Tuc[̍ŏ̉摜t@CԂ
	function Resource:getFirstImage()
		return self:foreachDescendant(
			function(i, v)
				if v:hasImageLoader() then return v end
			end
		)
	end
	
	-- Tuc[̍Ō̉摜t@CԂ
	function Resource:getLastImage()
		return self:foreachReverseDescendant(
			function(i, v)
				if v:hasImageLoader() then return v end
			end
		)
	end
	
	-- w̃pXĕԂ
	function Resource:findPath( path )
		return self:foreachReverseDescendant(
			function(i, v)
				if v:path() == path then return v end
			end
		)
	end
	
	-- \[g
	-- ver0.2.7bŃ\[gĂn悤ɏC
	function Resource.class:sortResources( list, comp )
		-- ftHg̔rpN[W
--~ 		defaultComp = function( lval, rval )
--~ 			-- nil nꍇi炵j
--~ 			if lval == nil then return false end
--~ 			if rval == nil then return true end
--~ 			-- fBNgDŕׂ
--~ 			if lval:isDirectory() then
--~ 				if not rval:isDirectory() then return true end
--~ 			elseif rval:isDirectory() then
--~ 				return false
--~ 			end
--~ 			-- OŕёւiWindows W̕ёւ֐gj
--~ 			return ( app.compareString( lval:name(), rval:name() ) == -1 )
--~ 		end
--~ 		
--~ 		comp = comp or defaultComp
--~ 		list:sort( comp )
	end

	
	-------------------------------------------------------------
	-- A[JCu֘A
	-------------------------------------------------------------
	-- ̃\[X܂ރA[JCut@C
	function Resource:containerFile()
		return nil
	end

	-- A[JCũTuc[ǂݍށiĂяopj
	function Resource:_loadArchivedTreeSub( childrenList, loader, progressCallback )

		if childrenList == nil then return false end
				
		-- ̃t@C̃Xg쐬
		local list = Array()
		local containerFile = self
		if self:containerFile() and not self:hasArchiveLoader() then
			containerFile = self:containerFile()
		end

		local idx, childInfo
		for idx, childInfo in childrenList do
			-- A[JCu
			local archivedInfo = {}
			archivedInfo.containerFile = containerFile
			archivedInfo.buffer = function( file )
				local buf = loader:getFile( file )
				if buf == nil then
					-- [hs
					local name = ""
					if containerFile then containerFile:name() end
					app.messageBox( "A[JCut@C̓WJɎs܂B\n"..name,
						"WJG[", "OK");
				end
				return buf
			end
			archivedInfo.isDirectory = childInfo.isDirectory
			archivedInfo.compsize = childInfo.compsize
			archivedInfo.lastUpdateTime = childInfo.lastUpdateTime
			archivedInfo.filesize = childInfo.filesize
			archivedInfo.position = childInfo.position
			archivedInfo.crc = childInfo.crc
			
			-- t@CIuWFNg쐬
			local child = Resource:newArchivedFile( childInfo, archivedInfo )
			if child then
				-- Xgɂ̃t@Cǉ
				list:add( child )
				
				-- fBNgȂATuc[쐬
				if child:isDirectory() then
					child:_loadArchivedTreeSub( childInfo.children, loader, progressCallback )
				end
			end

			-- vOXpR[obNĂяo
			if progressCallback then progressCallback() end
		end
		
		-- 쐬̃t@CXg\[g
--~ 		Resource:sortResources( list )
		
		-- Xg̃t@CTuc[ƂĒǉ
		local startTime = app.getTicks()
		list:foreach(
			function ( i, v )
				self:addNode( v )
			end
		)
		print( "Tuc[쐬IF"..( (app.getTicks() - startTime) / 1000 ).."b" )
		
		return true
	end

	-- A[JCũTuc[ǂݍ
	function Resource:loadArchivedTree( childrenList, loader, fileCount )
		print( "loadArchivedTree : "..self:name() )
		if childrenList == nil then return false end
		if fileCount == 0 then
			app.setStatusText( "A[JCuɑΉt@C܂ł" )
			return true
		end
		local startTime = app.getTicks()
		
		-- vOXEChȄ
		local progressWindow
		local progressCallback = nil
		
		local count = 0
		progressWindow = app.createProgressWindow()

		progressCallback = function()
			count = count + 1
			progressWindow:setProgressPos( ( count / fileCount ) * 100 )
			progressWindow:pumpMessages()
		end

		progressWindow:setTitle( "t@CXg쐬" )
		progressWindow:setMessage( self:name() )
		progressWindow:show()

		local ret = self:_loadArchivedTreeSub( childrenList, loader, progressCallback )
		
		-- vOXEChE
		if progressWindow then
			progressWindow:close()
		end
		
		-- Xe[^Xo[XV
		local count = self:getTotalMarkCount()
		local sec =  (app.getTicks() - startTime) / 1000
		if count == 0 then
			app.setStatusText( "A[JCuɑΉt@C܂ł" )
		else
			app.setStatusText( "A[JCu"..self:getTotalMarkCount().."̑Ήt@C܂" )
		end
		
		print( "t@CXg쐬ԁF"..sec.."b" )
		return ret
	end


	-------------------------------------------------------------
	-- NX\bh
	-------------------------------------------------------------
	-- t@CVXẽt@CIuWFNg쐬ĕԂ
	function Resource.class:newFSFile( fileInfo )
		-- ׂt@CmF
		if self:isFileToIgnore( fileInfo ) then
			return nil
		end
		-- fBNg̏ꍇ
		if fileInfo.isDirectory then
			return Directory( fileInfo.filepath )
		end
		-- t@C̏ꍇ
		return File( fileInfo.filepath )
	end

	-- A[JCũt@CIuWFNg쐬ĕԂ
	function Resource.class:newArchivedFile( fileInfo, archivedInfo )
		-- ׂt@CmF
		if self:isFileToIgnore( fileInfo ) then
			return nil
		end
		-- fBNg̏ꍇ
		if fileInfo.isDirectory then
			return ArchivedDirectory( fileInfo.filepath, archivedInfo )
		end
		-- t@C̏ꍇ
		return ArchivedFile( fileInfo.filepath, archivedInfo )
	end

	-- ׂt@CȂ true Ԃ
	function Resource.class:isFileToIgnore( fileInfo )
		if fileInfo == nil then return true end
		if fileInfo.isFile then
			-- t@C^Cvo^Ȃgq̏ꍇ͒ǉȂ
			local fileType = FileType:findFileType( fileInfo.filepath )
			if fileType == nil then return true end
		end
		return false
	end
	
	-- fVACY
	function Resource.class:deserialize( data, basePath )
		local class = _G[data.class]
		if not LuaClass.isClass(class) then return end
		local res = class:new()
		res:deserialize( data, basePath )
		return res
	end

Class:new( Resource )


-------------------------------------------------------------------------
-- ۃt@CNX
-------------------------------------------------------------------------
AbstractFile = { "AbstractFile", "Resource"; class = {}; }

	function AbstractFile:init( name, path )
		self:super( name, path )
	end
	
	-- t@CpXݒ
	function AbstractFile:setPath( path )
		self:super( path )
		
		if path then
			-- t@C^Cvݒ
			self.mFileType = FileType:findFileType( path )
			if self.mFileType then
				self.mImageLoader = self.mFileType:firstImageLoader()
				self.mArchiveLoader = self.mFileType:firstArchiveLoader()
			end
		else
			self.mFileType = nil
			self.mImageLoader = nil
			self.mArchiveLoader = nil
		end
		self.mSize = 0				-- t@CTCY
		self.mbLoadFail = nil		-- [hstOiO񃍁[hɎsĂꍇtruej
		self.mContainerFile = nil	-- ̃t@C܂ރA[JCut@C
		self.mExpandInfo = nil		-- A[JCuWJp

		-- m[hJEgp̃}[N
		if self:hasImageLoader() then
			self:setMark( true )
		else
			self:setMark( nil )
		end
	end

	-------------------------------------------------------------
	-- {IȃvpeB
	-------------------------------------------------------------
	-- t@C^Cg
	function AbstractFile:fileTitle()
	end

	-- t@Cgq
	function AbstractFile:fileExtension()
		local title, extension = app.splitExtension( self:name() )
		return extension
	end

	-- t@CTCY
	function AbstractFile:size()
		return self.mSize
	end

	-- f
	-- cx, cy, bpp = resource:pixelInfo()
	function Resource:pixelInfo()
		return self.mPixelWidth or 0, self.mPixcelHeight or 0, self.mBpp or 0
	end

	-- t@C^CvԂ
	function AbstractFile:fileType()
		return self.mFileType
	end

	-- t@CVXẽt@CpX
	-- erɑ݂Ȃt@CȂ nil ܂͈ꎞt@CԂ
	function AbstractFile:fsFilePath()
		return nil
	end

	-- t@CH
	function AbstractFile:isFile()
		return true
	end

	-- Kwꂽ\[XH
	function AbstractFile:hasSubTree()
		return self:hasArchiveLoader()
	end
	
	-- Tuc[͓ǂݍ܂Ă邩
	function AbstractFile:isSubTreeLoaded()
		return self:getNodeCount() ~= 0
	end

	-------------------------------------------------------------
	-- t@C̃[h֘A
	-------------------------------------------------------------
	-- ̃t@Cǂݍ񂾃obt@Ԃ
	function AbstractFile:buffer( callback )
		return nil
	end

	-- ̃t@C܂ރA[JCut@C
	function AbstractFile:containerFile()
		return nil
	end


	-------------------------------------------------------------
	-- [_[
	-------------------------------------------------------------
	-- ̃t@C̉摜[_[܂ nil Ԃ
	function AbstractFile:imageLoader()
		return self.mImageLoader
	end

	-- ̃t@C̃A[JCu[_[܂ nil Ԃ
	function AbstractFile:archiveLoader()
		return self.mArchiveLoader
	end


	-- ̃t@C̉摜[_[݂΁Atrue Ԃ
	function AbstractFile:hasImageLoader()
		return self.mImageLoader ~= nil
	end

	-- ̃t@C̃A[JCu[_[݂΁Atrue Ԃ
	function AbstractFile:hasArchiveLoader()
		return self.mArchiveLoader ~= nil
	end

	-- ̃t@C̉摜[_[݂΁A摜Ԃ
	function AbstractFile:imageInfo()
		local fileType = self:fileType()
		if fileType == nil then return nil end
		local loader
		for loader in fileType:imageLoaders() do
			local info = loader:getPictureInfo( self )
			if info then return info end
		end
		return nil
	end

	-- ̃t@C̃A[JCu[_[݂΁At@CXgԂ
	-- QԖڂ̖߂lɑt@CARԖڂ̖߂lɎgp[_[Ԃ
	function AbstractFile:archiveFileList()
		local fileType = self:fileType()
		if fileType == nil then return nil end
		local firstErr, loader
		for loader in fileType:archiveLoaders() do
			local list, fileCount, err = loader:getFileList( self )
			firstErr = firstErr or err
			if not err then return list, fileCount, loader end
		end
		
		if firstErr ~= nil then
			-- [hs
			local name = self:name()
			app.messageBox( "A[JCut@C̓WJɎs܂B\n"..name.."\n"..firstErr,
				"WJG[", "OK");
		else
			-- ̃t@C^Cvp̃[_[
		end
		
		return nil, fileCount, loader
	end

	-- Tuc[ǂݍ
	function AbstractFile:loadSubTree()
		return false
	end

Class:new( AbstractFile )


-------------------------------------------------------------------------
-- t@C
-- t@CVXeA܂̓A[JCut@C̃t@C
-------------------------------------------------------------------------
File = { "File", "AbstractFile"; class = {}; }

	-- path : t@CpX
	-- archivedInfo : A[JCut@C̏ꍇ͂̏e[u
	function File:init( path, archivedInfo )
		-- ϐ
		self:super()
		if path then
			self:setPath( path )
			self:setFSFileInfo()
		end
	end

	-- t@CVXẽt@C擾
	function File:setFSFileInfo()
		local fsPath = self:fsFilePath()
		if fsPath == nil then return end
		
		local info = app.getFileInfo( fsPath )
		if info == nil then
			print( "t@CǂݍݎsF"..fsPath )
			self.mbLoadFail = true	-- [hstO
		else
			self.mSize = info.filesize
			self.mCreateTime = info.createTime
			self.mLastUpdateTime = info.lastUpdateTime
			self.mLoader = nil		-- [_[
			self.mbLoadFail = nil	-- [hstO
		end
		self.mExpandInfo = nil
	end

	-------------------------------------------------------------
	-- VACY
	-------------------------------------------------------------
	-- VACY
	-- stream   : io.open() ŊJꂽt@Cnh
	-- basePath : ΃pXŏoꍇ̓x[XpXB΃pX̏ꍇ nil
	-- indent   : Cfg
	function File:serialize( stream, basePath, indent )
		local tab = "  "
		local spc = string.rep(tab, indent)
		stream:write(spc..'{ class="File";\n')
		stream:write(spc..tab..'name='..app.safeLiteral(self:name())..';\n')
		stream:write(spc..tab..'path='..app.safeLiteral(self:path())..';\n')
		stream:write(spc..'};\n')
	end

	-- fVACY
	function File:deserialize( data, basePath )
		-- e[u珉
		if basePath then data.path = app.joinPath( basePath, data.path ) end
		print( "pXF"..data.path )
		self:setName( data.name )
		self:setPath( data.path )
		self:setFSFileInfo()
	end

	-------------------------------------------------------------
	-- {IȃvpeB
	-------------------------------------------------------------
	-- t@CVXẽt@CpX
	-- erɑ݂Ȃt@CȂ nil ܂͈ꎞt@CԂ
	function File:fsFilePath()
		return self:path()
	end

	-- t@CH
	function File:isFile()
		return true
	end


	-------------------------------------------------------------
	-- t@C̃[h֘A
	-------------------------------------------------------------
	-- ̃t@Cǂݍ񂾃obt@Ԃ
	function File:buffer()
		return app.createFileBuffer( self:path() )
	end

	-- ̃t@C܂ރA[JCut@C
	function File:containerFile()
		return nil
	end


	-------------------------------------------------------------
	-- [_[
	-------------------------------------------------------------

	-- Tuc[ǂݍ
	function File:loadSubTree()
		-- Tuc[Ȃ^ǂݍݍς݂̏ꍇ͖߂
		if not self:hasSubTree() or self:isSubTreeLoaded() then
			return false
		end

		-- A[JCũt@C񃊃Xg擾
		local childList, fileCount, loader = self:archiveFileList()
		local bOK =  self:loadArchivedTree( childList, loader, fileCount )
		return bOK
	end


Class:new( File )



-------------------------------------------------------------------------
-- A[JCut@C
-------------------------------------------------------------------------
ArchivedFile = { "ArchivedFile", "AbstractFile"; class = {}; }

	-- path : t@CpX
	-- archivedInfo : A[JCut@C̏ꍇ͂̏e[u
	function ArchivedFile:init( path, archivedInfo )
		-- ϐ
		self:super()
		if path then self:setPath( path ) end
		if archivedInfo then self:setArchiveInfo( archivedInfo ) end
	end

	-- A[JCu擾
	function ArchivedFile:setArchiveInfo( archivedInfo )
		if archivedInfo == nil then return end
		
		self.mContainerFile = archivedInfo.containerFile
		self.buffer = archivedInfo.buffer
		self.mCreateTime = archivedInfo.lastUpdateTime
		self.mLastUpdateTime = archivedInfo.lastUpdateTime
		self.mSize = archivedInfo.filesize

		-- A[JCuWJp
		self.mExpandInfo = {}
		self.mExpandInfo.position = archivedInfo.position
		self.mExpandInfo.filesize = archivedInfo.filesize
		self.mExpandInfo.compsize = archivedInfo.compsize
		self.mExpandInfo.crc = archivedInfo.crc

		self.mbLoadFail = nil		-- [hstO
	end

	-------------------------------------------------------------
	-- VACY
	-------------------------------------------------------------
	-- fVACY
	function ArchivedFile:deserialize( data, basePath )
		-- e[u珉
--~ 		if basePath then data.path = app.joinPath( basePath, data.path ) end
--~ 		self:setName( data.name )
--~ 		self:setPath( data.path )
--~ 		self:setFSFileInfo()
	end

	-- VACY
	-- stream   : io.open() ŊJꂽt@Cnh
	-- basePath : ΃pXŏoꍇ̓x[XpXB΃pX̏ꍇ nil
	-- indent   : Cfg
	function ArchivedFile:serialize( stream, basePath, indent )
--~ 		local tab = "  "
--~ 		local spc = string.rep(tab, indent)
--~ 		stream:write(spc..'{ class="ArchivedFile";\n')
--~ 		stream:write(spc..tab..'name='..self:name()..';\n')
--~ 		stream:write(spc..tab..'path='..self:path()..';\n')
--~ 		stream:write(spc..tab..'archivedInfo={\n')
--~ 		stream:write(spc..tab..tab..'\n')
--~ 		stream:write(spc..tab..'};\n')
--~ 		stream:write(spc..'};\n')
	end


	-------------------------------------------------------------
	-- {IȃvpeB
	-------------------------------------------------------------
	-- t@CVXẽt@CpX
	-- erɑ݂Ȃt@CȂ nil ܂͈ꎞt@CԂ
	function ArchivedFile:fsFilePath()
		return nil
	end

	-- t@CH
	function ArchivedFile:isFile()
		return true
	end


	-------------------------------------------------------------
	-- t@C̃[h֘A
	-------------------------------------------------------------
	-- ̃t@Cǂݍ񂾃obt@Ԃ
	function ArchivedFile:buffer()
		return app.createFileBuffer( self:path() )
	end

	-- ̃t@C܂ރA[JCut@C
	function ArchivedFile:containerFile()
		return self.mContainerFile
	end


	-------------------------------------------------------------
	-- [_[
	-------------------------------------------------------------

	-- Tuc[ǂݍ
	function ArchivedFile:loadSubTree()
		if not self:hasSubTree() or self:isSubTreeLoaded() then
			return false
		end

		-- A[JCũt@C񃊃Xg擾
		local childList, fileCount, loader = self:archiveFileList()
		local bOK =  self:loadArchivedTree( childList, loader, fileCount )
		return bOK
	end


Class:new( ArchivedFile )


-------------------------------------------------------------------------
-- ۃfBNgNX
-------------------------------------------------------------------------
AbstractDirectory = { "AbstractDirectory", "Resource"; class = {}; }

	function AbstractDirectory:init( name, path )
		self:super( name, path )
		self.mFileType = nil			-- t@C^Cv
		self.mIsSubTreeLoaded = false	-- qc[͓ǂݍ܂Ă邩
	end

	-- fBNgH
	function AbstractDirectory:isDirectory()
		return true
	end

	-- Kwꂽ\[XH
	function AbstractDirectory:hasSubTree()
		return true
	end
	
	-- Tuc[ǂݍ񂾂ݒ
	function AbstractDirectory:setSubTreeLoaded( bLoaded )
		self.mIsSubTreeLoaded = bLoaded
	end

	-- Tuc[͓ǂݍ܂Ă邩
	function AbstractDirectory:isSubTreeLoaded()
		return self.mIsSubTreeLoaded
	end


Class:new( AbstractDirectory )


-------------------------------------------------------------------------
-- t@CVXẽfBNg
-------------------------------------------------------------------------
Directory = { "Directory", "AbstractDirectory"; class = {}; }

	function Directory:init( path )
		self:super( nil, path )
		self.mFileType = nil			-- t@C^Cv
	end

	-- Tuc[t@CVXeǂݍ
	function Directory:loadSubTree()
		if not self:hasSubTree() or self:isSubTreeLoaded() then
			return false
		end

		-- ̃t@C̃Xg쐬
		local startTime = app.getTicks()
		local list = Array()

		-- t@CǉpN[W
		local createChild = function(i, v)
			-- Jg^efBNg͒ǉȂ
			if v == "." or v == ".." then return end

			-- t@C擾
			local childInfo = app.getFileInfo( app.joinPath(self:path(), v) )
			
			-- t@CIuWFNg쐬
			local file = Resource:newFSFile( childInfo )
			if file then
				list:add( file )
			end
		end
		
		-- t@CVXeォ璼̃Tuc[ǂݍ
		local searchPath = app.joinPath( self:path(), "*" )
		local children = app.findFile( searchPath )
		if children then
			-- ̃Tuc[쐬
			table.foreach( children, createChild )			
			
			-- 쐬̃t@CXg\[g
--~ 			Resource:sortResources( list )
			
			-- Xg̃t@CTuc[ƂĒǉ
			list:foreach(
				function ( i, v ) self:addNode( v ) end
			)
		end
		
		-- Xe[^Xo[XV
		local count = self:getTotalMarkCount()
		local sec =  (app.getTicks() - startTime) / 1000
		if count == 0 then
			app.setStatusText( "fBNgɑΉt@C܂ł" )
		else
			app.setStatusText( "fBNg"..self:getTotalMarkCount().."̑Ήt@C܂" )
		end
		
		-- ǂݍݏI
		print( "t@CXg쐬ԁF"..sec.."b" )
		self.mIsSubTreeLoaded = true
		self:setSubTreeOpenFlag( true )
		return true
	end

	-------------------------------------------------------------
	-- VACY
	-------------------------------------------------------------
	-- VACY
	-- stream   : io.open() ŊJꂽt@Cnh
	-- basePath : ΃pXŏoꍇ̓x[XpXB΃pX̏ꍇ nil
	-- indent   : Cfg
	function Directory:serialize( stream, basePath, indent )
		local tab = "  "
		local spc = string.rep(tab, indent)
		stream:write(spc..'{ class="Directory";\n')
		stream:write(spc..tab..'name='..app.safeLiteral(self:name())..';\n')
		stream:write(spc..tab..'path='..app.safeLiteral(self:path())..';\n')
		stream:write(spc..tab..'treeOpened='..tostring(self:isSubTreeOpened())..';\n')
		stream:write(spc..tab..'isLoaded='..tostring(self:isSubTreeLoaded())..';\n')
--~ 		stream:write(spc..tab..'children={\n')
--~ 		self:foreachChild(
--~ 			function ( i, res )
--~ 				res:serialize( stream, basePath, indent + 2 )
--~ 			end
--~ 		)
--~ 		stream:write(spc..tab..'};\n')
		stream:write(spc..'};\n')
	end

	-- fVACY
	function Directory:deserialize( data, basePath )
		-- e[u珉
		if basePath then data.path = app.joinPath( basePath, data.path ) end
		print( "pXF"..data.path )
		self:setPath( data.path )
		if data.isLoaded then
			self:loadSubTree()
			self:setSubTreeOpenFlag( data.treeOpened )
		end
--~ 		if data.children == nil then return end
--~ 		table.foreach( data.children,
--~ 			function (i, child)
--~ 				local res = Resource:deserialize( child, basePath )
--~ 				self:addNode( res )
--~ 			end
--~ 		)
	end


Class:new( Directory )


-------------------------------------------------------------------------
-- A[JCũfBNg
-------------------------------------------------------------------------
ArchivedDirectory = { "ArchivedDirectory", "AbstractDirectory"; class = {}; }

	function ArchivedDirectory:init( path, archivedInfo )
		self:super( filename, path )
		self.mArchivedInfo = archivedInfo
		if archivedInfo then
			self.mContainerFile = archivedInfo.containerFile
		end
	end

	-- ̃fBNg܂ރA[JCut@C
	function ArchivedDirectory:containerFile()
		return self.mContainerFile
	end

	-- Tuc[͓ǂݍ܂Ă邩
	function ArchivedDirectory:isSubTreeLoaded()
		return true
	end

	-- A[JCũTuc[ǂݍ
	function ArchivedDirectory:loadArchivedTree( childrenList, loader, fileCount )
		if self.mArchivedInfo == nil then return false end
		self.mIsSubTreeLoaded = self:super( childrenList, loader, fileCount )
		return self.mIsSubTreeLoaded
	end

	-------------------------------------------------------------
	-- VACY
	-------------------------------------------------------------
	-- VACY
	-- stream   : io.open() ŊJꂽt@Cnh
	-- basePath : ΃pXŏoꍇ̓x[XpXB΃pX̏ꍇ nil
	-- indent   : Cfg
	function ArchivedDirectory:serialize( stream, basePath, indent )
	end

	-- fVACY
	function ArchivedDirectory:deserialize( data, basePath )
	end


Class:new( ArchivedDirectory )


-------------------------------------------------------------------------
-- ޗp̘_tH_
-------------------------------------------------------------------------
Folder = { "Folder", "AbstractDirectory"; class = {}; }

	function Folder:init( name )
		self:super( name )
	end

	-- Kwꂽ\[XH
	function Folder:hasSubTree()
		return true
	end

	-- Tuc[͓ǂݍ܂Ă邩
	function Folder:isSubTreeLoaded()
		return true
	end

	-------------------------------------------------------------
	-- VACY
	-------------------------------------------------------------
	-- VACY
	-- stream   : io.open() ŊJꂽt@Cnh
	-- basePath : ΃pXŏoꍇ̓x[XpXB΃pX̏ꍇ nil
	-- indent   : Cfg
	function Folder:serialize( stream, basePath, indent )
		local tab = "  "
		local spc = string.rep(tab, indent)
		stream:write(spc..'{ class="Folder";\n')
		stream:write(spc..tab..'name='..app.safeLiteral(self:name())..';\n')
		stream:write(spc..tab..'treeOpened='..tostring(self:isSubTreeOpened())..';\n')
		stream:write(spc..tab..'children={\n')
		self:foreachChild(
			function ( i, res )
				res:serialize( stream, basePath, indent + 2 )
			end
		)
		stream:write(spc..tab..'};\n')
		stream:write(spc..'};\n')
	end

	-- fVACY
	function Folder:deserialize( data, basePath )
		-- e[u珉
		self:setName( data.name )
		self:setSubTreeOpenFlag( data.treeOpened )
		if data.children == nil then return end
		table.foreach( data.children,
			function (i, child)
				local res = Resource:deserialize( child, basePath )
				self:addNode( res )
			end
		)
	end

Class:new( Folder )



-------------------------------------------------------------------------
-- end of file
-------------------------------------------------------------------------
end_declare()