-------------------------------------------------------------------------
-- test of LuaUnit
-------------------------------------------------------------------------
LUA_PATH = "./?.lua;./LuaClass/?.lua"
require "LuaUnit"

-- Class name declarations.
begin_declare()
declare("WasRun")
declare("DerivedWasRun")
declare("TestCaseTest")


-------------------------------------------------------------------------
-- WasRun
-------------------------------------------------------------------------

WasRun = { "WasRun", "TestCase"; }

	function WasRun:init( ... )
		self:super( unpack(arg) )
	end

	function WasRun:setUp()
		self.log = "setUp "
	end

	function WasRun:tearDown()
		self.log = self.log .. "tearDown "
	end

	function WasRun:testMethod()
		self.log = self.log .. "testMethod "
	end

	function WasRun:testBrokenMethod()
		error( "Broken method called (for test)." )
	end
	
	
	
	function WasRun:testAssertEqualsFail()
		assertEquals( 1, 2 )
	end

	function WasRun:testAssertNotEqualsFail()
		assertNotEquals( 2, 2 )
	end

	function WasRun:testAssertSameFail()
		assertSame( self, nil )
	end

	function WasRun:testAssertNotSameFail()
		assertNotSame( self, self )
	end

	function WasRun:testAssertTrueFail()
		assertTrue( false )
	end

	function WasRun:testAssertFalseFail()
		assertFalse( true )
	end


Class:new ( WasRun )



-------------------------------------------------------------------------
-- @class DerivedWasRun
-------------------------------------------------------------------------
DerivedWasRun = { "DerivedWasRun", "WasRun"; }

	function DerivedWasRun:testMethod()
		self:super()
	end
	
	function DerivedWasRun:testNotInheritedTestFail()
		assertEquals( 1, 2 )
	end


Class:new( DerivedWasRun )



-------------------------------------------------------------------------
-- TestCaseTest
-------------------------------------------------------------------------

TestCaseTest = { "TestCaseTest", "TestCase"; }

	function TestCaseTest:setUp()
		self.result = TestResult()
	end

	function TestCaseTest:tearDown()
		print( self.result:summary() )
	end

	function TestCaseTest:testTemplateMethod()
		local test = WasRun( "testMethod" )
		test:run(self.result)
		assertEquals( "setUp testMethod tearDown ", test.log )
	end

	function TestCaseTest:testResult()
		local test = WasRun("testMethod")
		test:run(self.result)
		assertEquals( "1 runs, 0 errors, 0 failures", self.result:summary() )
	end

	function TestCaseTest:testFailedResult()
		local test = WasRun("testBrokenMethod")
		test:run(self.result)
		assertEquals("1 runs, 1 errors, 0 failures", self.result:summary() )
	end

	function TestCaseTest:testFailedResultFormatting()
		self.result:testStarted()
		self.result:addFailure()
		assertEquals( "1 runs, 0 errors, 1 failures", self.result:summary() )
	end

	function TestCaseTest:testSuite()
		local suite = TestSuite()
		suite:add( WasRun("testMethod") )
		suite:add( WasRun("testBrokenMethod") )
		suite:run(self.result)
		assertEquals( "2 runs, 1 errors, 0 failures", self.result:summary() )
	end

	function TestCaseTest:testAssert()
		local suite = TestSuite()
		suite:add( WasRun("testAssertEqualsFail") )
		suite:add( WasRun("testAssertNotEqualsFail") )
		suite:add( WasRun("testAssertSameFail") )
		suite:add( WasRun("testAssertNotSameFail") )
		suite:add( WasRun("testAssertTrueFail") )
		suite:add( WasRun("testAssertFalseFail") )
		suite:run(self.result)
		assertEquals( "6 runs, 0 errors, 6 failures", self.result:summary() )
	end

	function TestCaseTest:testDerivedTest()
		local suite = TestSuite()
		suite:add( DerivedWasRun("testMethod") )
		suite:add( DerivedWasRun("testNotInheritedTestFail") )
		suite:run(self.result)
		assertEquals( "2 runs, 0 errors, 1 failures", self.result:summary() )
	end


Class:new ( TestCaseTest )



-------------------------------------------------------------------------
-- run test.
-------------------------------------------------------------------------

local result = TestResult()
local suite = TestSuite( TestCaseTest )


suite:run( result )
print "==================================================="
print( "Errors logged for the "..suite:getName().." test:" )
local i
for i=1, result:errorCount() do
	print( result:errors():at(i) )
end
print "==================================================="
print( "Failures logged for the "..suite:getName().." test:" )
local i
for i=1, result:failureCount() do
	print( result:failures():at(i) )
end
print "==================================================="
print( "Summary of "..suite:getName().." test:" )
print( result:summary() )


-------------------------------------------------------------------------
-- end of file
-------------------------------------------------------------------------
end_declare()